/**
 * @file main.c
 * @author WHXY
 * @brief
 * @version 0.1
 * @date 2021-09-22
 *
 * @copyright Copyright (c) 2021
 *
 */
/*******************************************************************************
 *
 * 代码许可和免责信息
 * 武汉芯源半导体有限公司授予您使用所有编程代码示例的非专属的版权许可，您可以由此
 * 生成根据您的特定需要而定制的相似功能。根据不能被排除的任何法定保证，武汉芯源半
 * 导体有限公司及其程序开发商和供应商对程序或技术支持（如果有）不提供任何明示或暗
 * 含的保证或条件，包括但不限于暗含的有关适销性、适用于某种特定用途和非侵权的保证
 * 或条件。
 * 无论何种情形，武汉芯源半导体有限公司及其程序开发商或供应商均不对下列各项负责，
 * 即使被告知其发生的可能性时，也是如此：数据的丢失或损坏；直接的、特别的、附带的
 * 或间接的损害，或任何后果性经济损害；或利润、业务、收入、商誉或预期可节省金额的
 * 损失。
 * 某些司法辖区不允许对直接的、附带的或后果性的损害有任何的排除或限制，因此某些或
 * 全部上述排除或限制可能并不适用于您。
 *
 *******************************************************************************/
/******************************************************************************
 * Include files
 ******************************************************************************/
#include "main.h"
#include "cw32l052_systick.h"
#include "cw32l052_rcc.h"
#include "cw32l052_gpio.h"
#include "cw32l052_adc.h"

/******************************************************************************
 * Local pre-processor symbols/macros ('#define')
 ******************************************************************************/

/******************************************************************************
 * Global variable definitions (declared in header file with 'extern')
 ******************************************************************************/
uint16_t gAdcResult[16];
volatile uint8_t gFlagIrq;
uint8_t gCntEoc;

/******************************************************************************
 * Local type definitions ('typedef')
 ******************************************************************************/

/******************************************************************************
 * Local function prototypes ('static')
 ******************************************************************************/

/******************************************************************************
 * Local variable definitions ('static')                                      *
 ******************************************************************************/

/******************************************************************************
 * Local pre-processor symbols/macros ('#define')
 ******************************************************************************/

/*****************************************************************************
 * Function implementation - global ('extern') and local ('static')
 ******************************************************************************/

/**
 * @brief ADC I/O初始化
 *
 */
void ADC_PortInit(void)
{
	// 打开GPIO时钟
	REGBITS_SET(CW_SYSCTRL->AHBEN, SYSCTRL_AHBEN_GPIOA_Msk | SYSCTRL_AHBEN_GPIOB_Msk | SYSCTRL_AHBEN_GPIOC_Msk);
	// 打开ADC时钟
	REGBITS_SET(CW_SYSCTRL->APBEN2, SYSCTRL_APBEN2_ADC_Msk);

	// set PA00 as AIN0 INPUT
	PA00_ANALOG_ENABLE();
	// set PA01 as AIN1 INPUT
	PA01_ANALOG_ENABLE();
	// set PA02 as AIN2 INPUT
	PA02_ANALOG_ENABLE();
	// set PA05 as AIN5 INPUT
	PA05_ANALOG_ENABLE();
	// set PA07 as AIN7 INPUT
	PA07_ANALOG_ENABLE();
	// set PC04 as AIN8 INPUT
	PC04_ANALOG_ENABLE();
	// set PC05 as AIN9 INPUT
	PC05_ANALOG_ENABLE();
	// set PB01 as AIN11 INPUT
	PB01_ANALOG_ENABLE();
}

/**
 * @brief Main function of project
 *
 * @return int
 */
int main(void)
{
	ADC_InitTypeDef ADC_InitStructure = { 0 };
	ADC_AltrTypeDef ADC_AltrStructure = { 0 };

	// LED初始化
	LED_Init();

	// 配置ADC测试IO口
	ADC_PortInit();

	// ADC初始化
	ADC_StructInit(&ADC_InitStructure);
	// ADC工作时钟配置
	ADC_InitStructure.ADC_ClkDiv = ADC_Clk_Div16;
	ADC_InitStructure.ADC_SampleTime = ADC_SampTime10Clk;

	// ADC逐通道依次转换模式配置
	ADC_AltrStructure.ADC_RepeatEn = ADC_REPEAT_Disable;
	ADC_AltrStructure.ADC_AltrChmux = ADC_AltrCh0 | ADC_AltrCh1 | ADC_AltrCh2 | ADC_AltrCh5 | ADC_AltrCh7 |
		ADC_AltrCh8 | ADC_AltrCh9 | ADC_AltrCh11 | ADC_AltrVddaDiV3 | ADC_AltrTs | ADC_AltrVref1P2;
	ADC_AltrStructure.ADC_DMASOFEn = ADC_DMASOFDisable;
	ADC_AltrStructure.ADC_InitStruct = ADC_InitStructure;

	ADC_AltrModeCfg(&ADC_AltrStructure);
	ADC_ITConfig(ADC_IT_EOC | ADC_IT_EOS, ENABLE);
	ADC_ClearITPendingAll();
	ADC_EnableIrq(ADC_INT_PRIORITY);

	// ADC使能
	ADC_Enable();
	ADC_SoftwareStartConvCmd(ENABLE);

	gCntEoc = 0;
	while (1)
	{
		// 逐通道依次转换模式
		while (!(gFlagIrq & ADC_ISR_EOC_Msk))
			;
		gFlagIrq &= (~ADC_ISR_EOC_Msk);
		gAdcResult[gCntEoc++] = ADC_GetConversionValue();

		if (gFlagIrq & ADC_ISR_EOS_Msk)
		{
			gFlagIrq = 0;
			gCntEoc = 0;
			PA15_TOG();
		}
	}
}

/**
 * @brief LED I/O初始化
 *
 */
void LED_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure = { 0 };

	// 打开GPIO时钟
	REGBITS_SET(CW_SYSCTRL->AHBEN, SYSCTRL_AHBEN_GPIOA_Msk | SYSCTRL_AHBEN_GPIOC_Msk);

	/* Configure the GPIO_LED pin */
	GPIO_InitStructure.Pins = GPIO_PIN_10;
	GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_Init(CW_GPIOC, &GPIO_InitStructure);

	GPIO_InitStructure.Pins = GPIO_PIN_15;
	GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
	GPIO_Init(CW_GPIOA, &GPIO_InitStructure);

	// LEDs are off.
	PC10_SETLOW();
	PA15_SETLOW();
}

/******************************************************************************
 * EOF (not truncated)
 ******************************************************************************/
#ifdef USE_FULL_ASSERT
/**
 * @brief  Reports the name of the source file and the source line number
 *         where the assert_param error has occurred.
 * @param  file: pointer to the source file name
 * @param  line: assert_param error line source number
 * @return None
 */
void assert_failed(uint8_t *file, uint32_t line)
{
	/* USER CODE BEGIN 6 */
	/* User can add his own implementation to report the file name and line number,
	   tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
	/* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */
