/**
 * @file main.c
 * @author WHXY
 * @brief
 * @version 0.1
 * @date 2021-09-22
 *
 * @copyright Copyright (c) 2021
 *
 */
/*******************************************************************************
*
* 代码许可和免责信息
* 武汉芯源半导体有限公司授予您使用所有编程代码示例的非专属的版权许可，您可以由此
* 生成根据您的特定需要而定制的相似功能。根据不能被排除的任何法定保证，武汉芯源半
* 导体有限公司及其程序开发商和供应商对程序或技术支持（如果有）不提供任何明示或暗
* 含的保证或条件，包括但不限于暗含的有关适销性、适用于某种特定用途和非侵权的保证
* 或条件。
* 无论何种情形，武汉芯源半导体有限公司及其程序开发商或供应商均不对下列各项负责，
* 即使被告知其发生的可能性时，也是如此：数据的丢失或损坏；直接的、特别的、附带的
* 或间接的损害，或任何后果性经济损害；或利润、业务、收入、商誉或预期可节省金额的
* 损失。
* 某些司法辖区不允许对直接的、附带的或后果性的损害有任何的排除或限制，因此某些或
* 全部上述排除或限制可能并不适用于您。
*
*******************************************************************************/
/******************************************************************************
 * Include files
 ******************************************************************************/
#include "main.h"
#include "cw32l052_systick.h"
#include "cw32l052_rcc.h"
#include "cw32l052_gpio.h"
#include "cw32l052_adc.h"

/******************************************************************************
 * Local pre-processor symbols/macros ('#define')
 ******************************************************************************/


/******************************************************************************
 * Global variable definitions (declared in header file with 'extern')
 ******************************************************************************/
uint16_t valueAdc;
uint32_t valueAdcAcc;
volatile uint8_t gFlagIrq;
uint16_t gCntEoc = 0;
uint8_t cntSample;
uint16_t gAdcResult[2];

/******************************************************************************
 * Local type definitions ('typedef')
 ******************************************************************************/

/******************************************************************************
 * Local function prototypes ('static')
 ******************************************************************************/

/******************************************************************************
 * Local variable definitions ('static')                                      *
 ******************************************************************************/


/******************************************************************************
 * Local pre-processor symbols/macros ('#define')
 ******************************************************************************/

/*****************************************************************************
 * Function implementation - global ('extern') and local ('static')
 ******************************************************************************/

/**
 * @brief ADC I/O初始化
 *
 */
void ADC_PortInit(void)
{
    //打开GPIO时钟
    REGBITS_SET(CW_SYSCTRL->AHBEN, SYSCTRL_AHBEN_GPIOA_Msk);
    //打开ADC时钟
    REGBITS_SET(CW_SYSCTRL->APBEN2, SYSCTRL_APBEN2_ADC_Msk);

    //set PA02 as AIN2 INPUT
    PA02_ANALOG_ENABLE();
}

void ADCInit(void)
{
    ADC_InitTypeDef ADC_InitStructure = {0};
    ADC_SerialChTypeDef ADC_SerialChStructure= {0};

    ADC_PortInit();

    ADC_StructInit(&ADC_InitStructure);
    ADC_InitStructure.ADC_ClkDiv  = ADC_Clk_Div2;    // MCU 默认的频率为8MHz，使用内部2.5V为参考时，ADC的时钟最大为4MHz，故2分频
    ADC_InitStructure.ADC_VrefSel = ADC_Vref_BGR2p5;
    ADC_InitStructure.ADC_InBufEn = ADC_BufEnable;    // 打开输入缓冲
    ADC_SerialChStructure.ADC_Sqr0Chmux  = ADC_SqrVddaDiV3;
    ADC_SerialChStructure.ADC_Sqr1Chmux  = ADC_SqrCh2;
    ADC_SerialChStructure.ADC_SqrEns     = ADC_SqrEns01;
    ADC_SerialChStructure.ADC_DMASOFEn   = ADC_DMASOFDisable;
    ADC_SerialChStructure.ADC_InitStruct = ADC_InitStructure;

    /* 序列通道多次采样模式 */
    cntSample = 0x0A; //转换次数为0xFF + 1次.
    ADC_SerialChMoreModeCfg(&ADC_SerialChStructure, cntSample);
    ADC_ITConfig(ADC_IT_EOC | ADC_IT_EOA, ENABLE);
    ADC_EnableIrq(ADC_INT_PRIORITY);
    ADC_ClearITPendingAll();

    //ADC使能
    ADC_Enable();
    ADC_SoftwareStartConvCmd(ENABLE);

    while(1)
    {
        while(!(gFlagIrq&ADC_ISR_EOA_Msk));

        gFlagIrq = 0u;
        ADC_GetSqr0Result(gAdcResult);
        ADC_GetSqr1Result(&gAdcResult[1]);
        if(gCntEoc!=(cntSample+1))
        {
           // printf("ADC ERR\r\n");
            __breakpoint(0);
        }
        else
        {
            //valueAdcAcc = 120*gAdcResult[1]/gAdcResult[0]*2;
            valueAdcAcc = 3*2500*gAdcResult[0]/4096;
          //  printf("vbat:%d\r\n",valueAdcAcc);
        }
        gCntEoc=0u;
        //ADC 关闭
        ADC_SoftwareStartConvCmd(ENABLE);
      //  SysTickDelay(1000);
    }
}

/**
 * @brief Main function of project
 *
 * @return int
 */
int main(void)
{
    ADCInit();

}

/**
 * @brief LED I/O初始化
 *
 */
void LED_Init(void)
{
    GPIO_InitTypeDef GPIO_InitStructure = {0};

    //打开GPIO时钟
    REGBITS_SET(CW_SYSCTRL->AHBEN, SYSCTRL_AHBEN_GPIOA_Msk | SYSCTRL_AHBEN_GPIOC_Msk);

    /* Configure the GPIO_LED pin */
    GPIO_InitStructure.Pins = GPIO_PIN_10;
    GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_Init(CW_GPIOC, &GPIO_InitStructure);

    GPIO_InitStructure.Pins = GPIO_PIN_15;
    GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_Init(CW_GPIOA, &GPIO_InitStructure);

    //LEDs are off.
    PC10_SETLOW();
    PA15_SETLOW();
}

/******************************************************************************
 * EOF (not truncated)
 ******************************************************************************/
#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @return None
  */
void assert_failed(uint8_t* file, uint32_t line)
{
    /* USER CODE BEGIN 6 */
    /* User can add his own implementation to report the file name and line number,
       tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
    /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

