/**
 * @file main.c
 * @author WHXY
 * @brief
 * @version 0.1
 * @date 2023-07-21
 *
 * @copyright Copyright (c) 2022
 *
 */
/*******************************************************************************
*
* 代码许可和免责信息
* 武汉芯源半导体有限公司授予您使用所有编程代码示例的非专属的版权许可，您可以由此
* 生成根据您的特定需要而定制的相似功能。根据不能被排除的任何法定保证，武汉芯源半
* 导体有限公司及其程序开发商和供应商对程序或技术支持（如果有）不提供任何明示或暗
* 含的保证或条件，包括但不限于暗含的有关适销性、适用于某种特定用途和非侵权的保证
* 或条件。
* 无论何种情形，武汉芯源半导体有限公司及其程序开发商或供应商均不对下列各项负责，
* 即使被告知其发生的可能性时，也是如此：数据的丢失或损坏；直接的、特别的、附带的
* 或间接的损害，或任何后果性经济损害；或利润、业务、收入、商誉或预期可节省金额的
* 损失。
* 某些司法辖区不允许对直接的、附带的或后果性的损害有任何的排除或限制，因此某些或
* 全部上述排除或限制可能并不适用于您。
*
*******************************************************************************/
/******************************************************************************
 * Include files
 ******************************************************************************/
#include "main.h"
/******************************************************************************
 * Local pre-processor symbols/macros ('#define')
 ******************************************************************************/
//UARTx
#define  DEBUG_UARTx                   CW_UART2
#define  DEBUG_UART_CLK                RCC_APB1_PERIPH_UART2
#define  DEBUG_UART_APBClkENx          RCC_APBPeriphClk_Enable1
#define  DEBUG_UART_BaudRate           9600
#define  DEBUG_UART_UclkFreq           8000000

//UARTx GPIO
#define  DEBUG_UART_GPIO_CLK           (RCC_AHB_PERIPH_GPIOC | RCC_AHB_PERIPH_GPIOD)
#define  DEBUG_UART_TX_GPIO_PORT       CW_GPIOC
#define  DEBUG_UART_TX_GPIO_PIN        GPIO_PIN_12
#define  DEBUG_UART_RX_GPIO_PORT       CW_GPIOD
#define  DEBUG_UART_RX_GPIO_PIN        GPIO_PIN_2

//GPIO AF
#define  DEBUG_UART_AFTX               PC12_AFx_UART2TXD()
#define  DEBUG_UART_AFRX               PD02_AFx_UART2RXD()

//中断
#define  DEBUG_UART_IRQ                UART2_IRQn

/******************************************************************************
 * Global variable definitions (declared in header file with 'extern')
 ******************************************************************************/

/******************************************************************************
 * Local type definitions ('typedef')
 ******************************************************************************/

/******************************************************************************
 * Local function prototypes ('static')
 ******************************************************************************/
void RCC_Configuration(void);
void GPIO_Configuration(void);
void UART_Configuration(void);

/******************************************************************************
 * Local variable definitions ('static')                                      *
 ******************************************************************************/

/******************************************************************************
 * Local pre-processor symbols/macros ('#define')
 ******************************************************************************/

/*****************************************************************************
 * Function implementation - global ('extern') and local ('static')
 ******************************************************************************/

/**
 ***********************************************DEBUG_UARTx*******************************
 ** \brief  Main function of project
 **
 ** \return uint32_t return value, if needed
 **
 ******************************************************************************/
int32_t main(void)
{
    //配置RCC
    RCC_Configuration();

    //配置UART
    UART_Configuration();

    UART_SendString(DEBUG_UARTx, "\r\nCW32L052 UART AutoBaudRate\r\n");
    UART_SendString(DEBUG_UARTx, "\r\nPC发送以0xF8作为数据头的数据流\r\n");

    while(1)
    {
        //中断收发
    }
}

/**
 * @brief 配置RCC
 *
 */
void RCC_Configuration(void)
{
    //SYSCLK = HSI = 8MHz = HCLK = PCLK
    RCC_HSI_Enable(RCC_HSIOSC_DIV6);
}

/**
 * @brief 配置UART
 *
 */
void UART_Configuration(void)
{
    UART_InitTypeDef UART_InitStructure = {0};
    GPIO_InitTypeDef GPIO_InitStructure = {0};

    //时钟使能
    RCC_AHBPeriphClk_Enable(DEBUG_UART_GPIO_CLK, ENABLE);
    DEBUG_UART_APBClkENx(DEBUG_UART_CLK, ENABLE);

    //UART TX RX 复用
    DEBUG_UART_AFTX;
    DEBUG_UART_AFRX;

    GPIO_InitStructure.IT = GPIO_IT_NONE;
    GPIO_InitStructure.Pins = DEBUG_UART_TX_GPIO_PIN;
    GPIO_InitStructure.Mode = GPIO_MODE_OUTPUT_PP;
    GPIO_Init(DEBUG_UART_TX_GPIO_PORT, &GPIO_InitStructure);

    GPIO_InitStructure.Pins = DEBUG_UART_RX_GPIO_PIN;
    GPIO_InitStructure.Mode = GPIO_MODE_INPUT_PULLUP;
    GPIO_Init(DEBUG_UART_RX_GPIO_PORT, &GPIO_InitStructure);

    UART_InitStructure.UART_BaudRate = DEBUG_UART_BaudRate;
    UART_InitStructure.UART_Over = UART_Over_16;
    UART_InitStructure.UART_Source = UART_Source_PCLK;
    UART_InitStructure.UART_UclkFreq = DEBUG_UART_UclkFreq;
    UART_InitStructure.UART_StartBit = UART_StartBit_FE;
    UART_InitStructure.UART_StopBits = UART_StopBits_1;
    UART_InitStructure.UART_Parity = UART_Parity_No ;
    UART_InitStructure.UART_HardwareFlowControl = UART_HardwareFlowControl_None;
    UART_InitStructure.UART_Mode = UART_Mode_Rx | UART_Mode_Tx;
    UART_Init(DEBUG_UARTx, &UART_InitStructure);

    //优先级
    NVIC_SetPriority(DEBUG_UART_IRQ, 0);
    //UARTx中断使能
    NVIC_EnableIRQ(DEBUG_UART_IRQ);
    //使能UARTx TIMOV/BAUD中断
    UART_ITConfig(DEBUG_UARTx, UART_IT_TIMOV | UART_IT_BAUD, ENABLE);

    UART_SetAutoReload(DEBUG_UARTx, 0xFFFFFF); //TIMARR重载值
    UART_TimerModeConfig(DEBUG_UARTx, UART_TimerMode_AutoBaudRate1); //自动波特率检测模式1
}

/**
 * @brief UART2_IRQHandlerCallBack
 *        UART中断收发 0xF8
 */
void UART2_IRQHandlerCallBack(void)
{
    /* USER CODE BEGIN */
    static uint32_t cnt = 0;
    uint32_t temp = 0;
    uint8_t TxRxBuffer;
    if(UART_GetITStatus(DEBUG_UARTx, UART_IT_TIMOV)) //TIMARR溢出中断，记录溢出次数cnt
    {
        UART_ClearITPendingBit(DEBUG_UARTx, UART_IT_TIMOV);
        cnt++; 
    }
    if(UART_GetITStatus(DEBUG_UARTx, UART_IT_BAUD)) //自动波特率检测完成中断，修改波特率
    {
        temp = (16777216*cnt + UART_GetCounter(DEBUG_UARTx)) / 4;
        DEBUG_UARTx->BRRI = (uint16_t)(temp >> 4);
        DEBUG_UARTx->BRRF = (uint16_t)(temp & 0x0F);
        UART_ClearITPendingBit(DEBUG_UARTx, UART_IT_BAUD);
        UART_ClearITPendingBit(DEBUG_UARTx, UART_IT_RC);
        UART_ITConfig(DEBUG_UARTx, UART_IT_RC, ENABLE);//RC中断使能
        cnt = 0;
    }
    if(UART_GetITStatus(DEBUG_UARTx, UART_IT_RC) != RESET) //中断收发数据
    {
        UART_ClearITPendingBit(DEBUG_UARTx, UART_IT_RC);
        TxRxBuffer = UART_ReceiveData_8bit(DEBUG_UARTx);
        UART_SendData_8bit(DEBUG_UARTx, TxRxBuffer);
    }
    /* USER CODE END */
}

/******************************************************************************
 * EOF (not truncated)
 ******************************************************************************/
#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{
    /* USER CODE BEGIN 6 */
    /* User can add his own implementation to report the file name and line number,
       tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
    /* USER CODE END 6 */
}
#endif /* USE_FULL_ASSERT */

